/**
 *
 * Magnolia tag cloud module (http://www.openmindlab.com/lab/products/mgnltagcloud.html)
 * Copyright(C) 2010-2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnltagcloud.dialog;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.security.AccessDeniedException;
import info.magnolia.module.admininterface.dialogs.ConfiguredDialog;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sourceforge.openutils.mgnlcontrols.dialog.ConfigurableFreemarkerDialog;
import net.sourceforge.openutils.mgnltagcloud.bean.TagCloud;
import net.sourceforge.openutils.mgnltagcloud.manager.TagCloudManager;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author cstrappazzon
 * @version $Id: TagCloudDialog.java 2590 2010-05-30 16:50:28Z fgiust $
 */
public class TagCloudDialog extends ConfigurableFreemarkerDialog
{

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(ConfiguredDialog.class);

    /**
     * {@inheritDoc}
     */
    @Override
    protected String getPath()
    {
        return "dialog/tagcloud.ftl";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(HttpServletRequest request, HttpServletResponse response, Content websiteNode, Content configNode)
        throws RepositoryException
    {
        super.init(request, response, websiteNode, configNode);
        this.setConfig(
            "saveHandler",
            "net.sourceforge.openutils.mgnltagcloud.dialog.TagCloudDialogSafeMultivalueSaveHandler");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getValue()
    {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void addToParameters(Map<String, Object> parameters)
    {
        super.addToParameters(parameters);

        Collection<String> tagList = new ArrayList<String>();
        try
        {
            if (getStorageNode() != null
                && getStorageNode().getNodeData(this.getName()) != null
                && getStorageNode().getNodeData(this.getName()).getValues() != null)
            {
                for (Value value : getStorageNode().getNodeData(this.getName()).getValues())
                {
                    tagList.add(value.getString());
                }
            }
        }
        catch (AccessDeniedException e)
        {
            log.error("AccessDeniedException {}", e);
        }
        catch (RepositoryException e)
        {
            log.error("RepositoryException: {}", e);
        }

        // Get configured node
        TagCloud tagCloud = null;
        if (StringUtils.isNotBlank(this.getConfigValue("tagCloudName")))
        {
            TagCloud tagCloudOriginal = TagCloudManager.getInstance().getTagCloud(this.getConfigValue("tagCloudName"));
            if (tagCloudOriginal == null)
            {
                log.error("No tag cloud configured with name : {}", this.getConfigValue("tagCloudName"));
                parameters.put("configuredTagList", new String[]{});
                return;
            }
            try
            {
                tagCloud = (TagCloud) BeanUtils.cloneBean(tagCloudOriginal);
            }
            catch (IllegalAccessException e)
            {
                // should never happen
            }
            catch (InstantiationException e)
            {
                // should never happen
            }
            catch (InvocationTargetException e)
            {
                // should never happen
            }
            catch (NoSuchMethodException e)
            {
                // should never happen
            }
            tagCloud.setTags(new HashMap<String, Integer>());
        }
        else
        {
            tagCloud = new TagCloud();
            if (StringUtils.isNotBlank(this.getConfigValue("tagCloudRepository")))
            {
                tagCloud.setRepository(this.getConfigValue("tagCloudRepository"));
            }
            if (StringUtils.isNotBlank(this.getConfigValue("tagCloudPath")))
            {
                tagCloud.setPath(this.getConfigValue("tagCloudPath"));
            }
            if (StringUtils.isNotBlank(this.getConfigValue("tagCloudProperty")))
            {
                tagCloud.setPropertyName(this.getConfigValue("tagCloudProperty"));
            }
            else
            {
                // Set the actual node name, it's useful in some cases
                tagCloud.setPropertyName(this.getName());
            }
        }
        tagCloud.setCount(10000);
        TagCloudManager.getInstance().calculateTagCloud(tagCloud);

        Map<String, Map<String, Object>> tags = new HashMap<String, Map<String, Object>>(tagCloud.getTags().size());
        for (String key : tagCloud.getTags().keySet())
        {
            Map<String, Object> values = new HashMap<String, Object>(2);
            values.put("selected", tagList.contains(key));
            values.put("count", tagCloud.getTags().get(key));
            tags.put(key, values);
        }

        parameters.put("configuredTagList", tags);

        // load the script once: if there are multiple instances
        String jQueryAttributeName = "info.magnolia.cms.gui.dialog.jquery.loaded";
        boolean includejquery = getRequest().getAttribute(jQueryAttributeName) == null;
        parameters.put("includejquery", includejquery);
        if (includejquery)
        {
            getRequest().setAttribute(jQueryAttributeName, "true");
        }

        parameters.put("debugEnabled", Boolean.valueOf(this.getConfigValue("debugEnabled")));

        parameters.put("width", StringUtils.isNotEmpty(this.getConfigValue("width"))
            ? this.getConfigValue("width")
            : "");

    }
}
